<?php
/**
 * Copyright (C) 2010 Convio, Inc. All rights reserved.
 *
 * This program is free open source software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or (at your 
 * option) any later version. The GNU General Public License is available at 
 * http://www.gnu.org/licenses/. 
 *
 * =====================================================
 *
 * @file Single-sign-on (SSO), Constituent personalization implementation.
 * @version $Id: ConvioUser.inc,v 1.1 2010/01/26 00:13:03 mpih Exp $
 */

class ConvioAPIClient {

  function ConvioAPIClient() {}


  /**
   * Makes a POST API request to the Convio Cons Open API.
   *
   * @param $method The API method
   * @param $params An array of API parameters
   * @param $settings Optional API call settings
   *                  - response_code
   *                  - target
   * @return the SimpleXML object representing the success response, or FALSE if the API call failed
   */
  public static function post($method, $params = array(), $settings = array()) {
    $api_config = ConvioConfiguration::getInstance();
    if (! $api_config) {
      watchdog("convio", 
               "%method failed. The Convio Open API is not configured!", 
               array("%method"=>$method), 
               WATCHDOG_ERROR);
      return FALSE;
    }

    // Validate required fields: request.
    if (! $method) {
      watchdog("convio", 
               "No method was specified in call to Convio Open API!", 
               array(), 
               WATCHDOG_ERROR);
      return FALSE;
    }

    // Default values for optional settings.
    $response_code = $method . "Response";
    if (isset($settings["response_code"])) {
      $response_code = $settings["response_code"];
    }

    $target = ConvioConstants::$CR_CONS_API;
    if (isset($settings["target"])) {
      $target = $settings["target"];
    }

    $url = $api_config->getURI() . $target;

    $headers = array("Content-Type" => "application/x-www-form-urlencoded; charset=UTF-8");

    $base_params = array(
                         "api_key" => $api_config->getPublicKey(),
                         "v" => ConvioConstants::$CONS_API_VERSION,
                         "method" => $method,
                         );
    if ($target == ConvioConstants::$SR_CONS_API) {
      $base_params["login_name"] = $api_config->getUserName();
      $base_params["login_password"] = $api_config->getPassword();
    }

    $data = array_merge($params, $base_params);
    $query = http_build_query($data, NULL, "&");

    $http_result = drupal_http_request($url, $headers, "POST", $query);

    // The expected result for successful authentication is a 
    // 200 HTTP response with a <loginResponse> XML document.
    if (($http_result->code == 200 || $http_result->code == 403)
        && $http_result->headers["Content-Type"] == "text/xml; charset=UTF-8") {

      $xml = simplexml_load_string($http_result->data);
      if ($xml === false) {
        // Malformed XML.
        $msg = "Error parsing response for %url: %msg.";
        $vars = array("%url"=>$url, "%msg"=>$e->getMessage());
        watchdog("convio", $msg, $vars, WATCHDOG_ERROR);
        return FALSE;
      }

      if ($http_result->code == 200 && $xml->getName() == $response_code) {
        // A successful authentication response.

        return $xml;

      } else if ($http_result->code == 403 && $xml->getName() == "errorResponse") {
        // An error response.

        $msg = "Received error code %code from %url query=%query: %msg.";
        $vars = array("%query" => $query,
                      "%url" => $url, 
                      "%code" => (string) $xml->code, 
                      "%msg" => (string) $xml->message,
                      );
        watchdog("convio", $msg, $vars, WATCHDOG_NOTICE);

        return FALSE;
      }
    }

    // Unexpected response.
    $msg = "Received unexpected response %code from %url: %msg.";
    $vars = array("%code"=>$http_result->code, "%url"=>$url, "%msg"=>$http_result->data);
    watchdog("convio", $msg, $vars, WATCHDOG_ERROR);
    return FALSE;  
  }

}
