<?php
/**
 * Copyright (C) 2010 Convio, Inc. All rights reserved.
 *
 * This program is free open source software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or (at your 
 * option) any later version. The GNU General Public License is available at 
 * http://www.gnu.org/licenses/. 
 *
 * =====================================================
 *
 * @file The Convio integration configuration.
 * @version $Id: ConvioConfiguration.inc,v 1.2 2010/01/26 00:13:03 mpih Exp $
 */


/**
 * Convio configuration bean.
 */
class ConvioConfiguration {

  private static $instance = NULL;

  /**
   * Fetches the singleton ConvioConfiguration instance.
   */
  public static function getInstance() {
    if (self::$instance === NULL) {
      self::$instance = new ConvioConfiguration();
    }
    return self::$instance;
  }

  /**
   * Validates the API URI and key combination.
   *
   * @param $api_uri The API URI
   * @param $api_key The API key
   * @return true if the test API call worked, otherwise false
   */
  public static function test($api_uri, $api_key) {
    $url = $api_uri . ConvioConstants::$CR_CONS_API;
    $url .= "?api_key=" . urlencode($api_key);
    $url .= "&v=" . urlencode(ConvioConstants::$CONS_API_VERSION);

    // A 403 with errorResponse code 6 is the expected result of the test.
    // Anything else means that either the API URI or the API key is invalid.
    $http_result = drupal_http_request($url);

    if ($http_result->code == 403) {
      $xml = simplexml_load_string($http_result->data);
      if ($xml === false) {
        // Not a valid Cons API 403 error response. Assume something is wrong.
        return false;
      }

      if ($xml->getName() == "errorResponse" && $xml->code == 9 && 
          strpos($xml->message, "Missing parameter was: method") !== FALSE) {
        // Success. Expected the no_method error.
        return true;
      }
    }

    return false;
  }


  private $api_uri;
  private $api_key;
  private $api_skey;
  private $api_skey_location;
  private $api_user;
  private $api_password;


  function ConvioConfiguration() {
    $result = db_query("SELECT api_uri, api_key, api_skey, api_user, api_password FROM {convio}");
    $c = db_fetch_object($result);

    $this->api_uri = $c->api_uri;
    $this->api_key = $c->api_key;

    // Load secret key location and key.
    $this->setSecretKeyLocation($c->api_skey);

    $this->api_user = $c->api_user;
    $this->api_password = $c->api_password;
  }

  public function isEnabled() {
    if ($this->api_uri == null || $this->api_uri == "") {
      return FALSE;
    } else if ($this->api_key == null || $this->api_key == "") {
      return FALSE;
    } else if ($this->api_skey == null || $this->api_skey == "") {
      return FALSE;
    } else if ($this->api_user == null || $this->api_user == "") {
      return FALSE;
    } else if ($this->api_password == null || $this->api_password == "") {
      return FALSE;
    }
    return TRUE;
  }

  public function getURI() {
    return $this->api_uri;
  }

  public function setURI($uri) {
    $this->api_uri = $uri;
  }

  public function getPublicKey() {
    return $this->api_key;
  }

  public function setPublicKey($key) {
    $this->api_key = $key;
  }

  public function getSecretKey() {
    return $this->api_skey;
  }

  public function getSecretKeyLocation() {
    return $this->api_skey_location;
  }

  public function setSecretKeyLocation($skey) {
    if ($skey != null && $skey != "") {
      // skey only points to the location of the secret key file. 
      // Read the file contents to get real secret key.
      $fh = fopen($skey, 'r');
      $this->api_skey = trim(fgets($fh));
      fclose($fh);
    }

    $this->api_skey_location = $skey;
  }

  public function getUserName() {
    return $this->api_user;
  }

  public function setUserName($user) {
    $this->api_user = $user;
  }

  public function getPassword() {
    return $this->api_password;
  }

  public function setPassword($password) {
    $this->api_password = $password;
  }

  /**
   * Saves the API client configuration.
   */
  public function save() {
    db_query("DELETE FROM {convio}");

    db_query("INSERT INTO {convio} (api_uri, api_key, api_skey, api_user, api_password) VALUES ('%s', '%s', '%s', '%s', '%s')",
             $this->api_uri,
             $this->api_key,
             $this->api_skey_location,
             $this->api_user,
             $this->api_password);

    // Reset the singleton instance.
    self::$instance = NULL;
  }

}
