<?php
/**
 * Copyright (C) 2010 Convio, Inc. All rights reserved.
 *
 * This program is free open source software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or (at your 
 * option) any later version. The GNU General Public License is available at 
 * http://www.gnu.org/licenses/. 
 *
 * =====================================================
 *
 * @file Single-sign-on (SSO), Constituent personalization implementation.
 * @version $Id: ConvioUser.inc,v 1.1 2010/01/26 00:13:03 mpih Exp $
 */

class ConvioTokenHandler {

  private static $instance = NULL;  

  /**
   * Fetches the singleton ConvioTokenHandler instance.
   */
  public static function getInstance() {
    if (self::$instance === NULL) {
      self::$instance = new ConvioTokenHandler();
    }
    return self::$instance;
  }

  function ConvioTokenHandler() {}

  /**
   * Disables the caching of personalized content for this request.
   */
  public function disableCache() {
    global $convio_no_cache;
    $convio_no_cache = TRUE;
  }

  /**
   * Fetches the constituents basic attributes.
   *
   * @return an associative array of constituent attributes.
   */
  public function getValues() {
    $properties = NULL;

    $cons = ConvioLoginHandler::getInstance()->getUser();
    if ($cons != NULL) {
      // Authenticated user.

      // Use the cached C360 data unless the convio_no_cache flag has been set.
      // If convio_no_cache is set in the request, indicates that the C360 data should always be 
      // fetched fresh.
      $cache_key = "convio_cons_properties::" . $cons->getID();

      global $convio_no_cache;
      if (! isset($convio_no_cache) && isset($_SESSION[$cache_key])) {
        // Cached result.
        $c = $_SESSION[$cache_key];

        if ($c != NULL) {
          $cons = unserialize($c);
        }
      }

      $properties = $cons->getProperties();

      // A Drupal6 limitation (http://lists.drupal.org/pipermail/development/2009-December/034510.html)
      // prevents storing of the ConvioUser object in the session. So instead we serialize the data 
      // beforehand.
      $_SESSION[$cache_key] = serialize($cons);

      unset($convio_no_cache);
    }

    return $properties;
  }

  /**
   * Fetches the names and labels of available constituent fields.
   */
  public function getList() {

    $settings = array("target" => ConvioConstants::$SR_CONS_API,
                      "response_code" => "listConsFieldsResponse");

    $xml = ConvioAPIClient::post("listUserFields", array(), $settings);
    if ($xml !== FALSE) {
      foreach ($xml->field as $field) {
        $tokens[(string) $field->name] = t((string) $field->label);
      }
      watchdog("debug", print_r($tokens, TRUE));
    } else {
      throw new Exception("Failed to retrieve C360 constituent fields.");
    }

    $tokens["user"]["name"] = t("The user's name");
    return $tokens;
  }
}
