<?php
/**
 * Copyright (C) 2010 Convio, Inc. All rights reserved.
 *
 * This program is free open source software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or (at your 
 * option) any later version. The GNU General Public License is available at 
 * http://www.gnu.org/licenses/. 
 *
 * =====================================================
 *
 * @file The Convio integration configuration.
 * @version $Id: configure.inc,v 1.1 2010/01/25 21:31:43 mpih Exp $
 */


/**
 * Create the form for managing configuration.
 */
function convio_configuration_form($context) {
  $form['api_uri'] = array(
    '#type' => 'textfield',
    '#title' => t('API Service URI'),
    '#size' => 50,
    '#maxlength' => 200,
    '#description' => t('The Convio Open API service'),
    '#required' => TRUE,
  );

  $form['api_key'] = array(
    '#type' => 'textfield',
    '#title' => t('API Public Key'),
    '#size' => 30,
    '#maxlength' => 100,
    '#description' => t('The client-specific public key to access Convio Open APIs'),
    '#required' => TRUE,
  );

  $form['api_skey'] = array(
    '#type' => 'textfield',
    '#title' => t('API Secret Key File'),
    '#size' => 50,
    '#maxlength' => 100,
    '#description' => t('The file path to the client-specific secret key for encrypting Convio Open API calls'),
    '#required' => TRUE,
  );

  $form['api_user'] = array(
    '#type' => 'textfield',
    '#title' => t('API User'),
    '#size' => 30,
    '#maxlength' => 100,
    '#description' => t('The API user'),
    '#required' => TRUE,
  );

  $form['api_password'] = array(
    '#type' => 'textfield',
    '#title' => t('API Password'),
    '#size' => 30,
    '#maxlength' => 100,
    '#description' => t('The API password'),
    '#required' => TRUE,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  // Initialize the form.
  $c = ConvioConfiguration::getInstance();
  if ($c) {
    $form['api_uri']['#default_value'] = $c->getURI();
    $form['api_key']['#default_value'] = $c->getPublicKey();
    $form['api_skey']['#default_value'] = $c->getSecretKeyLocation();
    $form['api_user']['#default_value'] = $c->getUserName();
    $form['api_password']['#default_value'] = $c->getPassword();
  }

  return $form;
}

function convio_configuration_form_validate($form, &$form_state) {
  $api_uri = $form_state['values']['api_uri'];
  $api_key = $form_state['values']['api_key'];
  $api_skey = $form_state['values']['api_skey'];

  if (! valid_url($api_uri, true)) {
    form_set_error('api_uri', t('The API URI is not a valid URL.'));
    return;
  }

  // Validate the secret key file.
  $skey = FALSE;
  try {
    $fh = fopen($api_skey, 'r');
    $skey = trim(fgets($fh));
    fclose($fh);
  } catch (Exception $e) {
    form_set_error('api_skey', t('Unable to read the secret key file. Please check the file location and permissions: ' . $e->getMessage()));
    return;
  }
  if (! $skey) {
    form_set_error('api_skey', t('Invalid secret key file.'));
    return;
  }

  if (! ConvioConfiguration::test($api_uri, $api_key)) {
    form_set_error('api_key', t('The Open API configuration is not valid. Please check the URI and key, and make sure that API access has been enabled in the Convio application.'));
    return;
  }
}

function convio_configuration_form_submit($form, &$form_state) {
  $c = new ConvioConfiguration();
  $c->setURI($form_state['values']['api_uri']);
  $c->setPublicKey($form_state['values']['api_key']);
  $c->setSecretKeyLocation($form_state['values']['api_skey']);
  $c->setUserName($form_state['values']['api_user']);
  $c->setPassword($form_state['values']['api_password']);
  $c->save();

  $form_state['redirect'] = 'admin';
}
